///
/// \file ConstantBuffer.h
/// \brief Defines ConstantBuffer type.
///

#ifndef CONSTANT_BUFFER_H
#define CONSTANT_BUFFER_H

#include "DirectXHelper.h"

///
/// \brief Wrapper class for cbuffers that handles creation and updating
///  for a fixed type specified by the template parameter T.
///
template<typename T>
class ConstantBuffer
{
public:
	ConstantBuffer() : 
		mInitialized(false),
		mBuffer(0)
	{ 
	}

	~ConstantBuffer() 
	{
		ReleaseCOM(mBuffer); 
	}

	///
	/// Public structure instance mirroring the data stored in the constant buffer.
	///
	T Data;

	///
	/// Returns the d3d11 buffer this class wraps.  
	///
	ID3D11Buffer* Buffer()const 
	{
		return mBuffer;	
	}

	///
	/// Initializes the constant buffer.
	///
	void Initialize(ID3D11Device* device) 
	{
		HRESULT hr = S_OK;

		// Make constant buffer multiple of 16 bytes.
		D3D11_BUFFER_DESC desc;
		desc.Usage = D3D11_USAGE_DYNAMIC;
		desc.BindFlags = D3D11_BIND_CONSTANT_BUFFER;
		desc.CPUAccessFlags = D3D11_CPU_ACCESS_WRITE;
		desc.MiscFlags = 0;  
		desc.ByteWidth = static_cast<UINT>(sizeof(T) + (16 - (sizeof(T) % 16)));
		desc.StructureByteStride = 0;

		DX::ThrowIfFailed(device->CreateBuffer(&desc, 0, &mBuffer));

		mInitialized = true;
	}

	///
	/// Copies the system memory constant buffer data to the GPU constant buffer.
	/// This call should be made as infrequently as possible.
	///
	void ApplyChanges(ID3D11DeviceContext* dc)
	{
		_ASSERT(mInitialized);

		D3D11_MAPPED_SUBRESOURCE mappedResource;
 		dc->Map(mBuffer, 0, D3D11_MAP_WRITE_DISCARD, 0, &mappedResource);
		CopyMemory(mappedResource.pData, &Data, sizeof(T));
		dc->Unmap(mBuffer, 0);
	}

private:
	ConstantBuffer(const ConstantBuffer<T>& rhs);
	ConstantBuffer<T>& operator=(const ConstantBuffer<T>& rhs);

private:

	ID3D11Buffer* mBuffer;
	bool mInitialized;
};

#endif //CONSTANT_BUFFER_H